/* 
 * This file is part of the Track+ application, a 
 * tool for project and change management. 
 * 
 * Copyright (C) 2009 Trackplus 
 * 
 * Use and distribution of this software is governed by the 
 * Track+ license conditions. 
 * 
 * $Id: BudgetExpenseDatasourceBase.java 5103 2013-09-04 09:10:24Z tamas $ 
 * 
 */

package com.trackplus.plugin.report.accounting;		
import java.util.LinkedList;
import java.util.List;
import java.util.Locale;
import java.util.Map;

import com.aurel.track.admin.project.ProjectBL;
import com.aurel.track.admin.user.person.PersonBL;
import com.aurel.track.beans.TPersonBean;
import com.aurel.track.beans.TProjectBean;
import com.aurel.track.json.JSONUtility;
import com.aurel.track.plugin.DatasourceDescriptor;
import com.aurel.track.report.datasource.TimePeriodDatasource;
import com.aurel.track.util.GeneralUtils;
import com.aurel.track.util.IntegerStringBean;
import com.aurel.track.util.PropertiesHelper;


public abstract class BudgetExpenseDatasourceBase extends TimePeriodDatasource {

	public static interface BUGET_EXENSE_PARAMETER_NAME {
		public static String OVERVIEW = "overview";
		public static String OVERVIEW_OPTIONS = OVERVIEW + PARAMETER_NAME.OPTION_SUFFIX;
		public static String OVERVIEW_NAME_FIELD = OVERVIEW + PARAMETER_NAME.NAME_SUFFIX;
		public static String OVERVIEW_NAME_VALUE = PARAMETER_NAME.MAP_PREFIX + OVERVIEW;
		
		public static String PERSON = "person";
		public static String PERSON_OPTIONS = PERSON + PARAMETER_NAME.OPTION_SUFFIX;
		public static String PERSON_NAME_FIELD = PERSON + PARAMETER_NAME.NAME_SUFFIX;
		public static String PERSON_NAME_VALUE = PARAMETER_NAME.MAP_PREFIX + PERSON;
	}
	
	/** 
	 * The defined operation modes
	 * 0 - overview of own costs
	 * 1 - overview of all costs
	 */
	protected static class OperationMode {
		static public final int PERSONALOVERVIEW = 0;
		static public final int MANAGEROVERVIEW = 1;
	}
	
		
	/**
	 * In some cases the selectable projects are limited by extra rights
	 * @param personBean
	 * @return
	 */
	protected List<Integer> getSelectableBaseProjectIDs(TPersonBean personBean) {
		List<TProjectBean> projectBeanList = ProjectBL.loadProjectsWithReadIssueRight(personBean.getObjectID());
		return GeneralUtils.createIntegerListFromBeanList(projectBeanList);
	}
	
	/**
	 * Prepares a map for rendering the config page
	 * @param savedParamsMap
	 * @param datasourceDescriptor
	 * @param personBean
	 * @param locale 
	 * @return
	 */
	protected String getTimePeriodExtraParams(Map<String, Object> savedParamsMap,
			DatasourceDescriptor datasourceDescriptor,
			TPersonBean personBean, Locale locale) {
		StringBuilder stringBuilder = new StringBuilder();
		
		JSONUtility.appendIntegerValue(stringBuilder, BUGET_EXENSE_PARAMETER_NAME.OVERVIEW, 
				(Integer)savedParamsMap.get(BUGET_EXENSE_PARAMETER_NAME.OVERVIEW));
		JSONUtility.appendStringValue(stringBuilder, BUGET_EXENSE_PARAMETER_NAME.OVERVIEW_NAME_FIELD,
				BUGET_EXENSE_PARAMETER_NAME.OVERVIEW_NAME_VALUE);
		JSONUtility.appendILabelBeanList(stringBuilder, BUGET_EXENSE_PARAMETER_NAME.PERSON_OPTIONS, (List)PersonBL.loadPersons());
		JSONUtility.appendIntegerArrayAsArray(stringBuilder, BUGET_EXENSE_PARAMETER_NAME.PERSON,
				(Integer[])savedParamsMap.get(BUGET_EXENSE_PARAMETER_NAME.PERSON));
		JSONUtility.appendStringValue(stringBuilder, BUGET_EXENSE_PARAMETER_NAME.PERSON_NAME_FIELD, BUGET_EXENSE_PARAMETER_NAME.PERSON_NAME_VALUE);
		List<IntegerStringBean> operationModes = new LinkedList<IntegerStringBean>();
		String prefix = "accountingReportConfig.prompt.operationMode.";
		operationModes.add(new IntegerStringBean(prefix+OperationMode.PERSONALOVERVIEW,
				Integer.valueOf(OperationMode.PERSONALOVERVIEW)));
		operationModes.add(new IntegerStringBean(prefix+OperationMode.MANAGEROVERVIEW, 
				Integer.valueOf(OperationMode.MANAGEROVERVIEW)));
		JSONUtility.appendIntegerStringBeanList(stringBuilder, BUGET_EXENSE_PARAMETER_NAME.OVERVIEW_OPTIONS, operationModes, true);
		String budgetExpenseExtraParams = getBudgetExpenseExtraParams(savedParamsMap, datasourceDescriptor, personBean, locale);
		if (budgetExpenseExtraParams!=null && !"".equals(budgetExpenseExtraParams)) {
			stringBuilder.append(",").append(budgetExpenseExtraParams);
		}
		return stringBuilder.toString();
	}
	
	protected String getBudgetExpenseExtraParams(Map<String, Object> savedParamsMap,
			DatasourceDescriptor datasourceDescriptor,
			TPersonBean personBean, Locale locale) {
		return null;
	}

	/**
	 * 
	 * @param paramSettings
	 * @return
	 */
	protected Map<String, Object> loadParamObjectsFromPropertyStrings(String paramSettings) {
		Map<String, Object> paramsMap = super.loadParamObjectsFromPropertyStrings(paramSettings);
		if (paramSettings!=null) {
			Integer overview = PropertiesHelper.getIntegerProperty(paramSettings, BUGET_EXENSE_PARAMETER_NAME.OVERVIEW);
			if (overview==null) {
				overview = OperationMode.PERSONALOVERVIEW;
			}
			paramsMap.put(BUGET_EXENSE_PARAMETER_NAME.OVERVIEW, overview);
			Integer[] personID = PropertiesHelper.getIntegerArrProperty(paramSettings, BUGET_EXENSE_PARAMETER_NAME.PERSON);
			paramsMap.put(BUGET_EXENSE_PARAMETER_NAME.PERSON, personID);
		} else {
			paramsMap.put(BUGET_EXENSE_PARAMETER_NAME.OVERVIEW, OperationMode.PERSONALOVERVIEW);
		}
		return paramsMap;
	}

}

