/*
 * This file is part of the Track+ application, a tool for project and change
 * management.
 * 
 * Copyright (C) 2012 Trackplus
 * 
 * Use and distribution of this software is governed by the Track+ license
 * conditions.
 * 
 * $Id$
 * 
 * Author: <Tamas Ruff>
 */
package com.trackplus.plugin.report.meeting;

import java.util.Collections;
import java.util.Date;
import java.util.HashMap;
import java.util.LinkedList;
import java.util.List;
import java.util.Locale;
import java.util.Map;
import java.util.Set;
import java.util.SortedMap;
import java.util.TreeMap;

import org.apache.log4j.Logger;

import com.aurel.track.admin.customize.category.filter.tree.design.FilterUpperTO;
import com.aurel.track.admin.customize.category.report.execute.IDescriptionAttributes;
import com.aurel.track.admin.customize.category.report.execute.ReportExporter;
import com.aurel.track.admin.customize.lists.systemOption.IssueTypeBL;
import com.aurel.track.admin.customize.lists.systemOption.StatusBL;
import com.aurel.track.admin.customize.treeConfig.field.FieldDesignBL;
import com.aurel.track.admin.project.ProjectBL;
import com.aurel.track.admin.user.person.PersonBL;
import com.aurel.track.beans.TFieldBean;
import com.aurel.track.beans.TListTypeBean;
import com.aurel.track.beans.TPersonBean;
import com.aurel.track.beans.TProjectBean;
import com.aurel.track.beans.TStateBean;
import com.aurel.track.beans.TWorkItemBean;
import com.aurel.track.fieldType.constants.SystemFields;
import com.aurel.track.item.ItemBL2;
import com.aurel.track.item.ItemLoaderException;
import com.aurel.track.json.JSONUtility;
import com.aurel.track.linkType.ILinkType.PARENT_CHILD_EXPRESSION;
import com.aurel.track.persist.ReportBeanLoader;
import com.aurel.track.plugin.DatasourceDescriptor;
import com.aurel.track.report.datasource.IPluggableDatasource;
import com.aurel.track.report.datasource.ReportBeanDatasource;
import com.aurel.track.report.execute.ReportBean;
import com.aurel.track.report.execute.ReportBeans;
import com.aurel.track.util.DateTimeUtils;
import com.aurel.track.util.GeneralUtils;
import com.aurel.track.util.IntegerStringBean;
import com.aurel.track.util.PropertiesHelper;
import com.aurel.track.util.StringArrayParameterUtils;


/**
 * Datasource implemetation for Meetings
 * @author Tamas
 *
 */
public class MeetingDatasource extends ReportBeanDatasource {
	
	private static Logger logger = Logger.getLogger(MeetingDatasource.class);
	
	public static interface MEETING_PARAMETER_NAME {
		public static String MEETING = "meeting";
		public static String MEETING_OPTIONS = MEETING + PARAMETER_NAME.OPTION_SUFFIX;
		public static String MEETING_NAME_FIELD = MEETING + PARAMETER_NAME.NAME_SUFFIX;
		public static String MEETING_NAME_VALUE = PARAMETER_NAME.MAP_PREFIX + MEETING;
		
		public static String ONLY_ACTIVE_MEETINGS = "onlyActiveMeetings";
		public static String ONLY_ACTIVE_MEETINGS_FIELD = ONLY_ACTIVE_MEETINGS + PARAMETER_NAME.NAME_SUFFIX;
		public static String ONLY_ACTIVE_MEETINGS_VALUE = PARAMETER_NAME.MAP_PREFIX + ONLY_ACTIVE_MEETINGS;
		
		public static String ONLY_NOT_CLOSED_CHILDREN = "onlyNotClosedChildren";
		public static String ONLY_NOT_CLOSED_CHILDREN_FIELD = ONLY_NOT_CLOSED_CHILDREN + PARAMETER_NAME.NAME_SUFFIX;
		public static String ONLY_NOT_CLOSED_CHILDREN_VALUE = PARAMETER_NAME.MAP_PREFIX + ONLY_NOT_CLOSED_CHILDREN;
	}
	
	/**
	 * The name of the long text custom field for Participants
	 */
	private static String PARTICIPANTS_FIELD_NAME = "Participants";
	private static String FROMTOTIME_FIELD_NAME = "FromToTime";
	
	/**
	 * The issue type name of the meeting issue (hard coded to differentiate from the children of the meeting)
	 */
	private static String MEETING_ISSUETYPE = "MeetingIssue";
	
	public static interface MEETING_JASPER_PARAMETER_NAME {
		public static String PARTICIPANTS = "PARTICIPANTS";
		public static String MEETINGDATE = "MEETINGDATE";
		public static String NOTESBY = "NOTESBY";
		public static String MEETINGTITLE = "MEETINGTITLE";
		public static String FROMTOTIME = "FROMTOTIME";
	}
		
	/**
	 * Gets the data source object (a Document object in this case) retrieved using the parameters settings
	 * @param parameters
	 * @param datasourceDescriptor
	 * @param contextMap
	 * @param templateDescriptionMap
	 * @param templateID
	 * @param personBean
	 * @param locale
	 * @return
	 */
	public Object getDatasource(Map<String, String[]> parameters, DatasourceDescriptor datasourceDescriptor,
			Map<String, Object> contextMap, Map<String, Object> templateDescriptionMap,
			Integer templateID, TPersonBean personBean, Locale locale) {
		ReportBeans reportBeans = null;
		//Boolean fromIssueNavigator = (Boolean)contextMap.get(CONTEXT_ATTRIBUTE.FROM_ISSUE_NAVIGATOR);
		boolean useProjectSpecificID = false;
		Boolean projectSpecificID = (Boolean)contextMap.get(IPluggableDatasource.CONTEXT_ATTRIBUTE.USE_PROJETC_SPECIFIC_ID);
		if (projectSpecificID!=null) {
			useProjectSpecificID = projectSpecificID.booleanValue();
		}
		//Integer workItemID = (Integer)contextMap.get(CONTEXT_ATTRIBUTE.WORKITEMID);
		/*if (fromIssueNavigator!=null && fromIssueNavigator.booleanValue() && workItemID!=null) {
			//from issue navigator: contextmenu for a meeting on the issue navigator 
			if (workItemID!=null) {
				Set<Integer> childWorkItemIDsSet = ItemBL2.getChildHierarchy(
						new int[] {workItemID}, PARENT_CHILD_EXPRESSION.ALL_CHILDREN, null, null);
				childWorkItemIDsSet.add(workItemID);
				List<ReportBean> reportBeanList = ReportBeanLoader.getReportBeansByWorkItemIDs(
						GeneralUtils.createIntArrFromIntegerCollection(childWorkItemIDsSet), personBean.getObjectID(), locale);
				reportBeans= new ReportBeans(reportBeanList, personBean.getObjectID(), null, false);
			}
		} else {*/
				//from report configuration page: first save the parameters into the database, then get the reportBeans
				Map<String, Object> savedParamsMap = new HashMap<String, Object>();
				String paramSettings = loadParamObjectsAndPropertyStringsAndFromStringArrParams(parameters, locale, savedParamsMap);
				saveParameters(paramSettings, personBean.getObjectID(), templateID);
				Integer meetingID = (Integer)savedParamsMap.get(MEETING_PARAMETER_NAME.MEETING); 
				Boolean onlyActiveChildren = (Boolean)savedParamsMap.get(MEETING_PARAMETER_NAME.ONLY_NOT_CLOSED_CHILDREN);
				Integer direction = null;
				if (onlyActiveChildren==null || onlyActiveChildren.booleanValue()) {
					direction = PARENT_CHILD_EXPRESSION.ALL_NOT_CLOSED_CHILDREN;
				} else {
					direction = PARENT_CHILD_EXPRESSION.ALL_CHILDREN;
				}
				List<ReportBean> reportBeanList = null;
				if (meetingID!=null) {
					Set<Integer> linkedWorkItemIDsSet = ItemBL2.getChildHierarchy(
							new int[] {meetingID}, direction, null, null);
					linkedWorkItemIDsSet.add(meetingID);
					reportBeanList = ReportBeanLoader.getReportBeansByWorkItemIDs(
							GeneralUtils.createIntArrFromIntegerCollection(linkedWorkItemIDsSet),
							false, personBean.getObjectID(), locale);
					if (reportBeanList!=null) {
						//set a hardcoded issue type for the parent meeting
						for (ReportBean reportBean : reportBeanList) {
							TWorkItemBean workItemBean = reportBean.getWorkItemBean();
							Integer objectID = workItemBean.getObjectID();
							if (objectID.equals(meetingID)) {
								//FieldTypeManager.getFieldTypeName(SystemFields.INTEGER_ISSUETYPE)
								reportBean.getShowISOValuesMap().put(SystemFields.INTEGER_ISSUETYPE, MEETING_ISSUETYPE);
								break;
							}
						}
					}
				}
				reportBeans= new ReportBeans(reportBeanList, personBean.getObjectID(), null, false);
		//}
		String exportFormat = (String)contextMap.get(CONTEXT_ATTRIBUTE.EXPORT_FORMAT);
		if (exportFormat==null) {
			//not dynamically generated, get it from description
			if (templateDescriptionMap!=null) {
				exportFormat = (String)templateDescriptionMap.get(IDescriptionAttributes.FORMAT);
			}
		}
		boolean withHistory = false;
		Boolean history = false;
		if (templateDescriptionMap!=null) {
			if (templateDescriptionMap.get(IDescriptionAttributes.HISTORY)!=null) {
				try {
					history = Boolean.valueOf((String)templateDescriptionMap.get(IDescriptionAttributes.HISTORY));
					if (history!=null) {
						withHistory = history.booleanValue();
					}
				} catch (Exception e) {
					logger.warn("history should be a boolean string " + e.getMessage());
				}
			}
		}
		//if it will be exported to CSV or XLS then make the long texts plain, otherwise simplified HTML
		boolean longTextIsPlain = ReportExporter.FORMAT_CSV.equals(exportFormat) || 
			ReportExporter.FORMAT_XLS.equals(exportFormat) || 
			ReportExporter.FORMAT_XML.equals(exportFormat);
		return getDocumentFromReportBeans(reportBeans, withHistory,
			personBean, locale, null, null, longTextIsPlain, useProjectSpecificID);
	}
	
	/**
	 * Gets the extra parameters from the datasource
	 * @param params
	 * @param datasourceDescriptor
	 * @param contextMap
	 * @param templateDescriptionMap
	 * @param templateID
	 * @param personBean
	 * @param locale
	 * @return
	 */
	public Map<String, Object> getJasperReportParameters(Map<String, String[]> params, DatasourceDescriptor datasourceDescriptor,
			Map<String, Object> contextMap, Map<String, Object> templateDescriptionMap,
			Integer templateID, TPersonBean personBean, Locale locale) {
		Map<String, Object> parameters = new HashMap<String, Object>();
		Boolean fromIssueNavigator = (Boolean)contextMap.get(CONTEXT_ATTRIBUTE.FROM_ISSUE_NAVIGATOR);
		Integer meetingID = null;
		Integer workItemID = (Integer)contextMap.get(CONTEXT_ATTRIBUTE.WORKITEMID);
		if (fromIssueNavigator!=null && fromIssueNavigator.booleanValue() && workItemID!=null) {
			meetingID = workItemID;
		} else {
			meetingID = StringArrayParameterUtils.parseIntegerValue(params, MEETING_PARAMETER_NAME.MEETING); 
		}
		TWorkItemBean workItemBean = null;
		if (meetingID!=null) {
			try {
				workItemBean = ItemBL2.loadWorkItem(meetingID);
				ItemBL2.loadWorkItemCustomAttributes(workItemBean);
			} catch (ItemLoaderException e) {
				e.printStackTrace();
			}
			if (workItemBean!=null) {
				Date meetingDate = workItemBean.getEndDate();
				if (meetingDate==null) {
					meetingDate = workItemBean.getStartDate();
					if (meetingDate==null) {
						meetingDate = workItemBean.getCreated();
					}
				}
				parameters.put(MEETING_JASPER_PARAMETER_NAME.MEETINGDATE, meetingDate);
				Integer responsibleID = workItemBean.getResponsibleID();
				TPersonBean reponsiblePersonBean = PersonBL.loadByPrimaryKey(responsibleID);
				if (reponsiblePersonBean!=null) {
					parameters.put(MEETING_JASPER_PARAMETER_NAME.NOTESBY, reponsiblePersonBean.getLabel());
				}
				parameters.put(MEETING_JASPER_PARAMETER_NAME.MEETINGTITLE, workItemBean.getSynopsis());
				Integer participantFieldID = null;
				List<TFieldBean> participantsFieldBeans = FieldDesignBL.loadByName(PARTICIPANTS_FIELD_NAME);
				if (participantsFieldBeans!=null && participantsFieldBeans.size()>0) {
					participantFieldID = participantsFieldBeans.get(0).getObjectID();
					parameters.put(MEETING_JASPER_PARAMETER_NAME.PARTICIPANTS, workItemBean.getAttribute(participantFieldID));
				}
				Integer fromToTimeFieldID = null;
				List<TFieldBean> fromToTimeFieldBeans = FieldDesignBL.loadByName(FROMTOTIME_FIELD_NAME);
				if (fromToTimeFieldBeans!=null && fromToTimeFieldBeans.size()>0) {
					fromToTimeFieldID = fromToTimeFieldBeans.get(0).getObjectID();
					parameters.put(MEETING_JASPER_PARAMETER_NAME.FROMTOTIME, workItemBean.getAttribute(fromToTimeFieldID));
				}
			}
		}
		return parameters;
	}
	
	/**
	 * Gets the datasource parameters
	 * @param templateID
	 * @param datasourceDescriptor
	 * @param contextMap
	 * @param savedParamsMap
	 * @param personBean
	 * @param locale
	 * @return
	 */
	protected String getDatasourceParams(DatasourceDescriptor datasourceDescriptor,
			Map<String, Object> contextMap, Map<String, Object> savedParamsMap, TPersonBean personBean, Locale locale) {
		StringBuilder stringBuilder = new StringBuilder();
		Integer workItemID = (Integer)contextMap.get(CONTEXT_ATTRIBUTE.WORKITEMID);
		Integer meetingID = null;
		if (workItemID==null) {
			//previous meeting executed if any
			meetingID = (Integer)savedParamsMap.get(MEETING_PARAMETER_NAME.MEETING);
		} else {
			//report called by right clicking an issue in issue navigator
			meetingID = workItemID;
		}
		JSONUtility.appendStringValue(stringBuilder, MEETING_PARAMETER_NAME.MEETING_NAME_FIELD, MEETING_PARAMETER_NAME.MEETING_NAME_VALUE);
		Boolean onlyActiveMeetings = (Boolean)savedParamsMap.get(MEETING_PARAMETER_NAME.ONLY_ACTIVE_MEETINGS);
		if (onlyActiveMeetings==null) {
			onlyActiveMeetings = Boolean.TRUE;
		}
		JSONUtility.appendBooleanValue(stringBuilder, MEETING_PARAMETER_NAME.ONLY_ACTIVE_MEETINGS, onlyActiveMeetings.booleanValue());
		JSONUtility.appendStringValue(stringBuilder, MEETING_PARAMETER_NAME.ONLY_ACTIVE_MEETINGS_FIELD,
				MEETING_PARAMETER_NAME.ONLY_ACTIVE_MEETINGS_VALUE);
		
		Boolean onlyOpenChilderen = (Boolean)savedParamsMap.get(MEETING_PARAMETER_NAME.ONLY_NOT_CLOSED_CHILDREN);
		if (onlyOpenChilderen==null) {
			onlyOpenChilderen = Boolean.TRUE;
		}
		JSONUtility.appendBooleanValue(stringBuilder, MEETING_PARAMETER_NAME.ONLY_NOT_CLOSED_CHILDREN, onlyOpenChilderen.booleanValue());
		JSONUtility.appendStringValue(stringBuilder, MEETING_PARAMETER_NAME.ONLY_NOT_CLOSED_CHILDREN_FIELD,
				MEETING_PARAMETER_NAME.ONLY_NOT_CLOSED_CHILDREN_VALUE);
		List<IntegerStringBean> meetingOptions = new LinkedList<IntegerStringBean>();
		meetingID = getMeetingOptions(meetingID, personBean.getObjectID(), locale, onlyActiveMeetings.booleanValue(), meetingOptions);
		JSONUtility.appendIntegerValue(stringBuilder, MEETING_PARAMETER_NAME.MEETING, meetingID);
		JSONUtility.appendIntegerStringBeanList(stringBuilder,
				MEETING_PARAMETER_NAME.MEETING_OPTIONS, meetingOptions, true);
		return stringBuilder.toString();
	}
	
	/**
	 * Gets the meeting options
	 * @param personID
	 * @param locale
	 * @param onlyActiveMeetings
	 * @return
	 */
	private Integer getMeetingOptions(Integer currentMeetingID, Integer personID,
			Locale locale, boolean onlyActiveMeetings, List<IntegerStringBean> meetingOptions) {
		FilterUpperTO filterUpperTO = new FilterUpperTO();
		List<TProjectBean> usedProjects = ProjectBL.loadUsedProjectsFlat(personID);
		filterUpperTO.setSelectedProjects(GeneralUtils.createIntegerArrFromCollection(GeneralUtils.createIntegerListFromBeanList(usedProjects)));
		List<TListTypeBean> meetingIssueTypes = IssueTypeBL.loadByTypeFlag(TListTypeBean.TYPEFLAGS.MEETING);
		filterUpperTO.setSelectedIssueTypes(GeneralUtils.createIntegerArrFromCollection(GeneralUtils.createIntegerListFromBeanList(meetingIssueTypes)));
		if (onlyActiveMeetings) {
			List<TStateBean> activeStateBeans = StatusBL.loadActiveStates();
			filterUpperTO.setSelectedStates(GeneralUtils.createIntegerArrFromCollection(GeneralUtils.createIntegerListFromBeanList(activeStateBeans)));
		}
		//get and sort the meetings by date 
		List<TWorkItemBean> meetingWorkItems = ReportBeanLoader.getCustomFilterWorkItemBeans(filterUpperTO, personID, false);
		SortedMap<Date , List<IntegerStringBean>> meetingsByDate = new TreeMap<Date, List<IntegerStringBean>>();
		boolean meetingFound = false;
		if (meetingWorkItems!=null) {
			for (TWorkItemBean workItemBean : meetingWorkItems) {
				Integer workItemID = workItemBean.getObjectID();
				String title = workItemBean.getSynopsis();
				Date meetingDate = workItemBean.getEndDate();
				if (meetingDate==null) {
					meetingDate = workItemBean.getStartDate();
					if (meetingDate==null) {
						meetingDate = workItemBean.getCreated();
					}
				}
				if (currentMeetingID!=null && !meetingFound) {
					if (currentMeetingID.equals(workItemID)) {
						meetingFound = true;
					}
				}
				String meetingTitle = DateTimeUtils.getInstance().formatGUIDate(meetingDate, locale) + " " + title;
				List<IntegerStringBean> meetingsOnDate = meetingsByDate.get(meetingDate);
				if (meetingsOnDate==null) {
					meetingsOnDate = new LinkedList<IntegerStringBean>();
					meetingsByDate.put(meetingDate, meetingsOnDate);
				}
				meetingsOnDate.add(new IntegerStringBean(meetingTitle, workItemID));
			}
		}
		
		for (Date date : meetingsByDate.keySet()) {
			List<IntegerStringBean> meetingsOnDate = meetingsByDate.get(date);
			if (meetingsOnDate!=null) {
				if (meetingsOnDate.size()>1) {
					Collections.sort(meetingsOnDate);
				}
				meetingOptions.addAll(0, meetingsOnDate);
			}
		}
		if (meetingFound) {
			return currentMeetingID;
		} else {
			if (meetingOptions!=null && meetingOptions.size()>0) {
				return meetingOptions.get(0).getValue();
			}
			return null;
		}
	}
	
	/**
	 * 
	 * @param paramSettings
	 * @return
	 */
	protected Map<String, Object> loadParamObjectsFromPropertyStrings(String paramSettings) {
		Map<String, Object> paramsMap = new HashMap<String, Object>();
		if (paramSettings!=null) {
			Integer meetingID = PropertiesHelper.getIntegerProperty(paramSettings, MEETING_PARAMETER_NAME.MEETING);
			Boolean onlyOpenMeetings = PropertiesHelper.getBooleanProperty(paramSettings, MEETING_PARAMETER_NAME.ONLY_ACTIVE_MEETINGS);
			Boolean onlyOpenChildren = PropertiesHelper.getBooleanProperty(paramSettings, MEETING_PARAMETER_NAME.ONLY_NOT_CLOSED_CHILDREN);
			paramsMap.put(MEETING_PARAMETER_NAME.MEETING, meetingID);
			paramsMap.put(MEETING_PARAMETER_NAME.ONLY_ACTIVE_MEETINGS, onlyOpenMeetings);
			paramsMap.put(MEETING_PARAMETER_NAME.ONLY_NOT_CLOSED_CHILDREN, onlyOpenChildren);
		}
		return paramsMap;
	}
	
	/**
	 * Gets the parameter settings for report to be executed and the parameters to be saved 
	 * @param parameters parameters as String[]
	 * @param locale
	 * @param savedParamsMap output parameter: parameters transformed to the actual types
	 * @return
	 */
	protected String loadParamObjectsAndPropertyStringsAndFromStringArrParams(
			Map<String, String[]> params, Locale locale, Map<String, Object> savedParamsMap) {
		String paramSettings = "";
		Integer meeting = StringArrayParameterUtils.parseIntegerValue(params, MEETING_PARAMETER_NAME.MEETING);
		savedParamsMap.put(MEETING_PARAMETER_NAME.MEETING, meeting);
		Boolean onlyActiveMeetings = StringArrayParameterUtils.getBooleanValue(params, MEETING_PARAMETER_NAME.ONLY_ACTIVE_MEETINGS);
		if (onlyActiveMeetings==null) {
			onlyActiveMeetings = Boolean.FALSE;
		}
		savedParamsMap.put(MEETING_PARAMETER_NAME.ONLY_ACTIVE_MEETINGS, onlyActiveMeetings);
		Boolean onlyNotClosedChildren = StringArrayParameterUtils.getBooleanValue(params, MEETING_PARAMETER_NAME.ONLY_NOT_CLOSED_CHILDREN);
		if (onlyNotClosedChildren==null) {
			onlyNotClosedChildren = Boolean.FALSE;
		}
		savedParamsMap.put(MEETING_PARAMETER_NAME.ONLY_NOT_CLOSED_CHILDREN, onlyNotClosedChildren);
		paramSettings = PropertiesHelper.setIntegerProperty(paramSettings, MEETING_PARAMETER_NAME.MEETING, meeting);
		paramSettings = PropertiesHelper.setBooleanProperty(paramSettings, MEETING_PARAMETER_NAME.ONLY_ACTIVE_MEETINGS, onlyActiveMeetings);
		paramSettings = PropertiesHelper.setBooleanProperty(paramSettings, MEETING_PARAMETER_NAME.ONLY_NOT_CLOSED_CHILDREN, onlyNotClosedChildren);
		return paramSettings;
	}
	
	/**
	 * Refreshing of some parameters through ajax: only a part of the parameters should be recalculated 
	 * @param params
	 * @param templateID
	 * @param datasourceDescriptor
	 * @param contextMap
	 * @param personBean
	 * @param locale 
	 * @return
	 */
	@Override
	public String refreshParameters(Map<String, String[]> parameters, Integer templateID,
			DatasourceDescriptor datasourceDescriptor, Map<String, Object> contextMap,
			TPersonBean personBean, Locale locale) {
		Integer meeting = StringArrayParameterUtils.parseIntegerValue(parameters, MEETING_PARAMETER_NAME.MEETING);
		Boolean onlyActiveMeetings = StringArrayParameterUtils.getBooleanValue(parameters, MEETING_PARAMETER_NAME.ONLY_ACTIVE_MEETINGS);
		StringBuilder stringBuilder = new StringBuilder();
		stringBuilder.append("{");
		List<IntegerStringBean> meetingOptions = new LinkedList<IntegerStringBean>();
		meeting = getMeetingOptions(meeting,
				personBean.getObjectID(), locale, onlyActiveMeetings.booleanValue(), meetingOptions);
		if (meetingOptions!=null && meetingOptions.size()>0) {
			JSONUtility.appendIntegerValue(stringBuilder, MEETING_PARAMETER_NAME.MEETING_NAME_VALUE, meeting);
			JSONUtility.appendIntegerStringBeanList(stringBuilder, MEETING_PARAMETER_NAME.MEETING_OPTIONS, meetingOptions, true);
		}
		stringBuilder.append("}");
		return stringBuilder.toString();
	}
}
