/* 
 * This file is part of the Track+ application, a 
 * tool for project and change management. 
 * 
 * Copyright (C) 2010 Trackplus 
 * 
 * Use and distribution of this software is governed by the 
 * Track+ license conditions. 
 * 
 * $Id: StatusOverTimeBL.java 5103 2013-09-04 09:10:24Z tamas $ 
 * 
 * Author: <Tamas Ruff> 
 * 
 */
package com.trackplus.plugin.report.statusOverTime;

import java.util.Date;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Locale;
import java.util.Map;
import java.util.SortedMap;
import java.util.TreeMap;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.FactoryConfigurationError;
import javax.xml.parsers.ParserConfigurationException;

import org.apache.log4j.Logger;
import org.w3c.dom.Document;
import org.w3c.dom.Element;

import com.aurel.track.admin.customize.lists.systemOption.StatusBL;
import com.aurel.track.beans.TStateBean;
import com.aurel.track.report.dashboard.StatusOverTimeGraph;
import com.aurel.track.util.DateTimeUtils;

public class StatusOverTimeBL {
	private static Logger logger = Logger.getLogger(StatusOverTimeBL.class);
	static Document convertToDOM(SortedMap<Date, Map<Integer, Integer>> numberOfIssuesInStatusByDate, 
			List<Integer> statusIDs, String personName, int selectedTimeInterval, 
			boolean newIssues, String title, Locale locale) {
		Document dom = null;
		try{
			DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance ();
			DocumentBuilder builder = factory.newDocumentBuilder ();
			dom = builder.newDocument ();
		}catch (FactoryConfigurationError e){
			logger.error ("Creating the DOM document failed with FactoryConfigurationError:" + e.getMessage());
			return null;
		}catch (ParserConfigurationException e){
			logger.error ("Creating the DOM document failed with ParserConfigurationException: " + e.getMessage());
			return null;
		}
		Element root = dom.createElement ("track-report");
		Element createdBy =createDomElement("createdBy",personName,dom);
		root.appendChild(createdBy);
		Element createdAt =createDomElement("createdAt", DateTimeUtils.getInstance().formatISODateTime(new Date()),dom);
		root.appendChild(createdAt);
		Element etitle =createDomElement("title",title,dom);
		root.appendChild(etitle);
		String theType = null;
		if (newIssues) {
			theType="newIssues";
		} else {
			theType = "issuesInStatus";
		}
		Element dtype = createDomElement("dataseriesType",theType,dom);
		root.appendChild(dtype);		
		Element timeInterval =createDomElement("timeInterval",String.valueOf(selectedTimeInterval),dom);
		root.appendChild(timeInterval); 
		
		//Map<Integer, TStateBean> statusMap = GeneralUtils.createMapFromList(statusBeanList);
		
		Map<Integer, SortedMap<Date, Integer>> numberOfIssuesInDateByStatus = new HashMap<Integer, SortedMap<Date,Integer>>();
		Iterator<Date> itrIssuesInStatus = numberOfIssuesInStatusByDate.keySet().iterator();
		while (itrIssuesInStatus.hasNext()) {
			Date date = itrIssuesInStatus.next();
			Map<Integer, Integer> numberInStateMap = numberOfIssuesInStatusByDate.get(date); 
			if (numberInStateMap!=null && numberInStateMap.size()>0) {
				Iterator<Integer> itrStatus = numberInStateMap.keySet().iterator();
				while (itrStatus.hasNext()) {
					Integer statusID = itrStatus.next();
					Integer numberOfIssues = numberInStateMap.get(statusID);
					SortedMap<Date, Integer> numberOfIssuesInDate = numberOfIssuesInDateByStatus.get(statusID);
					if (numberOfIssuesInDate==null) {
						numberOfIssuesInDate = new TreeMap<Date, Integer>();
						numberOfIssuesInDateByStatus.put(statusID, numberOfIssuesInDate);
					}
					numberOfIssuesInDate.put(date, numberOfIssues);
				}
			}
		}
		if (newIssues) {
			Map<Date, Integer> numberOfIssuesInDate = numberOfIssuesInDateByStatus.get(StatusOverTimeGraph.ENTITY_PLACEHOLDER);
			if (numberOfIssuesInDate!=null) {
				//is it a selected status?
				Iterator<Date> itrDate = numberOfIssuesInDate.keySet().iterator();
				while (itrDate.hasNext()) {
					Date date = itrDate.next();
					Integer numberOfIssues = numberOfIssuesInDate.get(date);
					if (numberOfIssues!=null) {
						//String statusLabel = stateBean.getLabel();
						Element statusCount = dom.createElement ("s");
						statusCount.appendChild(createDomElement("label","new",dom));
						statusCount.appendChild(createDomElement("no",numberOfIssues.toString(),dom));
						statusCount.appendChild(createDomElement("date",DateTimeUtils.getInstance().formatISODateTime(date),dom));
						root.appendChild(statusCount);
					}
				}
			}
		} else {
			List<TStateBean> statusBeanList = (List)StatusBL.loadAll(locale);
			Iterator<TStateBean> itrIssuesAtDate = statusBeanList.iterator();
			while (itrIssuesAtDate.hasNext()) {
				TStateBean stateBean = itrIssuesAtDate.next();
				Integer statusID = stateBean.getObjectID();
				//TStateBean stateBean = statusMap.get(statusID);
				if (stateBean!=null) {
					Map<Date, Integer> numberOfIssuesInDate = numberOfIssuesInDateByStatus.get(statusID);
					if (numberOfIssuesInDate!=null) {
						//is it a selected status?
						Iterator<Date> itrDate = numberOfIssuesInDate.keySet().iterator();
						while (itrDate.hasNext()) {
							Date date = itrDate.next();
							Integer numberOfIssues = numberOfIssuesInDate.get(date);
							if (numberOfIssues!=null) {
								String statusLabel = stateBean.getLabel();
								Element statusCount = dom.createElement ("s");
								statusCount.appendChild(createDomElement("label",statusLabel,dom));
								statusCount.appendChild(createDomElement("no",numberOfIssues.toString(),dom));
								statusCount.appendChild(createDomElement("date",DateTimeUtils.getInstance().formatISODateTime(date),dom));
								root.appendChild(statusCount);
							}
						}
					}
				}
			}
		}
		dom.appendChild (root);
		return dom;
	}
	
	/**
	 * @param elementName
	 * @param elementValue
	 * @param dom
	 * @return
	 */
	private static Element createDomElement (String elementName, String elementValue, Document dom) {
		Element element = dom.createElement(elementName.replaceAll("\\s", ""));
		if (elementValue == null || "".equals(elementValue.trim())) {
			element.appendChild (dom.createTextNode (""));
		} else {
			element.appendChild (dom.createTextNode(elementValue));
		}
		return element;
	}
}
