/* 
 * This file is part of the Track+ application, a 
 * tool for project and change management. 
 * 
 * Copyright (C) 2010 Trackplus 
 * 
 * Use and distribution of this software is governed by the 
 * Track+ license conditions. 
 * 
 * $Id: StatusOverTimeDatasource.java 5103 2013-09-04 09:10:24Z tamas $ 
 * 
 * Author: <Tamas Ruff> 
 * 
 */
package com.trackplus.plugin.report.statusOverTime;

import java.io.OutputStream;
import java.util.Date;
import java.util.HashMap;
import java.util.LinkedList;
import java.util.List;
import java.util.Locale;
import java.util.Map;
import java.util.SortedMap;
import java.util.TreeMap;

import org.w3c.dom.Document;

import com.aurel.track.admin.customize.category.report.execute.ReportBeansToXML;
import com.aurel.track.admin.customize.lists.systemOption.StatusBL;
import com.aurel.track.beans.TPersonBean;
import com.aurel.track.beans.TWorkItemBean;
import com.aurel.track.json.JSONUtility;
import com.aurel.track.plugin.DatasourceDescriptor;
import com.aurel.track.report.dashboard.StatusOverTimeGraph;
import com.aurel.track.report.dashboard.StatusOverTimeGraph.CALCULATION_MODE;
import com.aurel.track.report.datasource.TimeIntervalWithStatusDatasource;
import com.aurel.track.util.GeneralUtils;
import com.aurel.track.util.IntegerStringBean;
import com.aurel.track.util.PropertiesHelper;
import com.aurel.track.util.StringArrayParameterUtils;

public class StatusOverTimeDatasource extends TimeIntervalWithStatusDatasource {
	
	
	
	public static interface STATUS_OVER_TIME_PARAMETER_NAME {
		/**
		 * Calculation modes
		 */
		public static String CALCULATION_MODE = "calculationMode";
		public static String CALCULATION_MODE_OPTIONS = CALCULATION_MODE + PARAMETER_NAME.OPTION_SUFFIX;
		public static String CALCULATION_MODE_NAME_FIELD = CALCULATION_MODE + PARAMETER_NAME.NAME_SUFFIX;
		public static String CALCULATION_MODE_NAME_VALUE = PARAMETER_NAME.MAP_PREFIX + CALCULATION_MODE;
	}
	
	/**
	 * Gets the data source object (a Document object in this case) retrieved using the parameters settings
	 * @param params
	 * @param datasourceDescriptor
	 * @param contextMap
	 * @param templateDescriptionMap
	 * @param templateID
	 * @param personBean
	 * @param locale
	 * @return
	 */
	public Object getDatasource(Map<String, String[]> parameters, DatasourceDescriptor datasourceDescriptor,
			Map<String, Object> contextMap, Map<String, Object> templateDescriptionMap,
			Integer templateID, TPersonBean personBean, Locale locale) {
		Map<String, Object> savedParamsMap = new HashMap<String, Object>();
		String paramSettings = loadParamObjectsAndPropertyStringsAndFromStringArrParams(parameters, locale, savedParamsMap);
		List<Integer> statusIDs;
		Integer[] statusIDsArr = (Integer[])savedParamsMap.get(TIME_INTERVAL_PARAMETER_NAME.STATUSES);
		//TODO title?
		String title = null;//StringArrayParameterUtils.getStringValue(configParameters, CONFIGURATION_PARAMETERS.TITLE);
		if (statusIDsArr==null || statusIDsArr.length==0) {
			statusIDs = GeneralUtils.createIntegerListFromBeanList(StatusBL.loadAll());
		} else {
		statusIDs = GeneralUtils.createListFromIntArr(statusIDsArr);
		}
		Date dateFrom = (Date)savedParamsMap.get(TIME_PERIOD_PARAMETER_NAME.DATE_FROM);
		Date dateTo = (Date)savedParamsMap.get(TIME_PERIOD_PARAMETER_NAME.DATE_TO);
		
		int selectedTimeInterval = (Integer)savedParamsMap.get(TIME_INTERVAL_PARAMETER_NAME.TIME_INTERVAL);
		int selectedCalculationMode = StringArrayParameterUtils.parseIntegerValue(parameters, 
				StatusOverTimeGraph.CONFIGURATION_PARAMETERS.SELECTED_CALCULATION_MODE, StatusOverTimeGraph.CALCULATION_MODE.NEW);
		paramSettings = PropertiesHelper.setIntegerProperty(paramSettings, StatusOverTimeGraph.CONFIGURATION_PARAMETERS.SELECTED_CALCULATION_MODE, selectedCalculationMode);
		saveParameters(paramSettings, personBean.getObjectID(), templateID);
		
		Integer datasourceType = (Integer)savedParamsMap.get(PARAMETER_NAME.DATASOURCETYPE);
		Integer projectOrReleaseID = (Integer)savedParamsMap.get(PARAMETER_NAME.PROJECT_OR_RELEASE_ID);
		Integer filterID = (Integer)savedParamsMap.get(PARAMETER_NAME.FILTER_ID);
		
		SortedMap<Integer, SortedMap<Integer, Map<Integer, Integer>>> yearToPeriodToEntityIDToNumbersMap = 
			new TreeMap<Integer, SortedMap<Integer, Map<Integer, Integer>>>();
		List<TWorkItemBean> workItemBeans = getWorkItemBeans(
				datasourceType, projectOrReleaseID, filterID, contextMap, true, true, true, personBean, locale);
		if (workItemBeans==null || workItemBeans.size()==0) {
			return null;
		}
		int[] workItemIDs = GeneralUtils.createIntArrFromIntegerArr(GeneralUtils.getBeanIDs(workItemBeans));
		boolean newIssues = false;	
		switch (selectedCalculationMode) {
		case StatusOverTimeGraph.CALCULATION_MODE.NEW:
			yearToPeriodToEntityIDToNumbersMap = StatusOverTimeGraph.calculateNewWorkItems(workItemBeans,
					dateFrom, dateTo, selectedTimeInterval);
			newIssues = true;
			break;
		case StatusOverTimeGraph.CALCULATION_MODE.ACTUAL_SAMPLE:
			yearToPeriodToEntityIDToNumbersMap = StatusOverTimeGraph.calculateTotalInStatus(workItemIDs,
					dateFrom, dateTo, statusIDs, selectedTimeInterval, locale);
					accumulateNumbers(yearToPeriodToEntityIDToNumbersMap);
			break;
		case StatusOverTimeGraph.CALCULATION_MODE.ACTUAL_ACTIVITY:
			yearToPeriodToEntityIDToNumbersMap = StatusOverTimeGraph.calculateStatus(workItemIDs,
					dateFrom, dateTo, statusIDs, selectedTimeInterval, locale);  		
			break;
		case StatusOverTimeGraph.CALCULATION_MODE.ACCUMULATED_ACTIVITY:
			yearToPeriodToEntityIDToNumbersMap = StatusOverTimeGraph.calculateStatus(workItemIDs,
					dateFrom, dateTo, statusIDs, selectedTimeInterval, locale);
			accumulateNumbers(yearToPeriodToEntityIDToNumbersMap);
			break;
		}
		SortedMap numberOfIssuesInStatusByDate = transformPeriodsToDates(yearToPeriodToEntityIDToNumbersMap, selectedTimeInterval);
		return StatusOverTimeBL.convertToDOM(numberOfIssuesInStatusByDate, statusIDs, personBean.getFullName(), selectedTimeInterval, newIssues, title, locale);

	}
	
	/**
	* Add the time series to the timeSeriesCollection
	* SortedMap at first and second level (year and period) 
	* (Sorted because the accumulated should be calculated in the right order) 
	* @param timeSeriesCollection
	* @param yearToPeriodToEntityIDToWorkItemNumbersMap
	* @param entityMap
	* @param selectedTimeInterval
	* @param accumulated
	*/
	private static void accumulateNumbers( 
			SortedMap<Integer, SortedMap<Integer, Map<Integer, Integer>>> yearToPeriodToEntityIDToWorkItemNumbersMap) {
		Map<Integer, Integer> accumulatedMap = new HashMap<Integer, Integer>();
		for (Integer year : yearToPeriodToEntityIDToWorkItemNumbersMap.keySet()) {
			SortedMap<Integer, Map<Integer, Integer>> intervalToStatusChangeBeans = yearToPeriodToEntityIDToWorkItemNumbersMap.get(year);
			for (Integer period : intervalToStatusChangeBeans.keySet()) {
				Map<Integer, Integer> entityIDToWorkItemNumbersMap = intervalToStatusChangeBeans.get(period);
				if (entityIDToWorkItemNumbersMap!=null) {
					for (Integer entityID : entityIDToWorkItemNumbersMap.keySet()) {
						Integer numberOfStates = entityIDToWorkItemNumbersMap.get(entityID);
						if (numberOfStates!=null) {
							Integer accumuletedValueForEntity = accumulatedMap.get(entityID);
							if (accumuletedValueForEntity==null) {
								accumulatedMap.put(entityID, numberOfStates);
								accumuletedValueForEntity = accumulatedMap.get(entityID);
							} else {
								accumuletedValueForEntity = Integer.valueOf(accumuletedValueForEntity.intValue()+numberOfStates.intValue());
								accumulatedMap.put(entityID, accumuletedValueForEntity);
							}
							entityIDToWorkItemNumbersMap.put(entityID, accumuletedValueForEntity);
						}
					}
				}
			}
		}
	}
	
	/**
	 * Serializes the datasource in an XML file
	 */
	public void serializeDatasource(OutputStream outputStream,
			Object datasource) {
		ReportBeansToXML.convertToXml(outputStream, (Document)datasource);
	}
	
	
	/**
	 * Prepares a map for rendering the config page
	 * @param savedParamsMap
	 * @param datasourceDescriptor
	 * @param personBean
	 * @param locale 
	 * @return
	 */
	protected String getTimeIntervalExtraParams(Map<String, Object> savedParamsMap,
			DatasourceDescriptor datasourceDescriptor,
			TPersonBean personBean, Locale locale) {
		StringBuilder stringBuilder = new StringBuilder();
		List<IntegerStringBean> calculationModes = new LinkedList<IntegerStringBean>();
		calculationModes.add(new IntegerStringBean("statusOverTime.calculationMode." + 
				StatusOverTimeGraph.getCalculationModeSuffix(CALCULATION_MODE.NEW),
				Integer.valueOf(CALCULATION_MODE.NEW)));
		calculationModes.add(new IntegerStringBean("statusOverTime.calculationMode." + 
				StatusOverTimeGraph.getCalculationModeSuffix(CALCULATION_MODE.ACTUAL_SAMPLE),
				Integer.valueOf(CALCULATION_MODE.ACTUAL_SAMPLE)));
		calculationModes.add(new IntegerStringBean("statusOverTime.calculationMode." + 
				StatusOverTimeGraph.getCalculationModeSuffix(CALCULATION_MODE.ACTUAL_ACTIVITY),
				Integer.valueOf(CALCULATION_MODE.ACTUAL_ACTIVITY)));
		calculationModes.add(new IntegerStringBean("statusOverTime.calculationMode." + 
				StatusOverTimeGraph.getCalculationModeSuffix(CALCULATION_MODE.ACCUMULATED_ACTIVITY),
				Integer.valueOf(CALCULATION_MODE.ACCUMULATED_ACTIVITY)));
		JSONUtility.appendIntegerStringBeanList(stringBuilder, STATUS_OVER_TIME_PARAMETER_NAME.CALCULATION_MODE_OPTIONS, calculationModes);
		JSONUtility.appendIntegerValue(stringBuilder, STATUS_OVER_TIME_PARAMETER_NAME.CALCULATION_MODE,
				(Integer)savedParamsMap.get(STATUS_OVER_TIME_PARAMETER_NAME.CALCULATION_MODE));
		JSONUtility.appendStringValue(stringBuilder, STATUS_OVER_TIME_PARAMETER_NAME.CALCULATION_MODE_NAME_FIELD,
				STATUS_OVER_TIME_PARAMETER_NAME.CALCULATION_MODE_NAME_VALUE);
		JSONUtility.appendILabelBeanList(stringBuilder, TIME_INTERVAL_PARAMETER_NAME.STATUS_OPTIONS,
				StatusBL.loadAll(locale));
		JSONUtility.appendIntegerArrayAsArray(stringBuilder, TIME_INTERVAL_PARAMETER_NAME.STATUSES,
				(Integer[])savedParamsMap.get(TIME_INTERVAL_PARAMETER_NAME.STATUSES));
		JSONUtility.appendStringValue(stringBuilder, TIME_INTERVAL_PARAMETER_NAME.STATUS_LABEL_KEY, "statusOverTime.prompt.statuses");
		JSONUtility.appendStringValue(stringBuilder, TIME_INTERVAL_PARAMETER_NAME.STATUS_NAME_FIELD,
				TIME_INTERVAL_PARAMETER_NAME.STATUS_NAME_VALUE, true);	
		return stringBuilder.toString();
		
	}
	
	/**
	 * 
	 * @param paramSettings
	 * @return
	 */
	protected Map<String, Object> loadParamObjectsFromPropertyStrings(String paramSettings) {
		Map<String, Object> paramsMap = super.loadParamObjectsFromPropertyStrings(paramSettings);
		Integer calculationMode = null;
		if (paramSettings!=null) {
			calculationMode = PropertiesHelper.getIntegerProperty(paramSettings, STATUS_OVER_TIME_PARAMETER_NAME.CALCULATION_MODE);			
		} 
		if (calculationMode==null) {
			calculationMode = CALCULATION_MODE.ACTUAL_SAMPLE;
		}
		paramsMap.put(STATUS_OVER_TIME_PARAMETER_NAME.CALCULATION_MODE, calculationMode);
		return paramsMap;
	}
}
